/*
 * File:   hardware.h
 * Author: Bogdan Kecman <bogdan.kecman@crsn.rs>
 *
 * Created on November 23, 2011, 4:24 PM
 * Licence: PUBLIC DOMAIN
 * 
 * $Id: hardware.h 234 2012-01-09 07:17:32Z arhimed $
 * $Rev: 234 $
 */

#ifndef __SID_GEN4_HARDWARE_H__
#define __SID_GEN4_HARDWARE_H__

#include <p18cxxx.h>
#include <delays.h>

//defined if detent is after both A and B rising edge 
//comment if detent is both after A and after B edge
#define DETENT_DOUBLE

//LCD pins (RW is tied to the ground)
#define PIN_LCD_D4          LATAbits.LATA2
#define PIN_LCD_D5          LATAbits.LATA3
#define PIN_LCD_D6          LATAbits.LATA4
#define PIN_LCD_D7          LATAbits.LATA5

#define PIN_LCD_E           LATBbits.LATB4
#define PIN_LCD_RS          LATBbits.LATB5
#define PIN_LCD_BRIGHTNESS  LATBbits.LATB3

#define TRIS_LCD_D4         TRISAbits.RA2
#define TRIS_LCD_D5         TRISAbits.RA3
#define TRIS_LCD_D6         TRISAbits.RA4
#define TRIS_LCD_D7         TRISAbits.RA5
#define TRIS_LCD_E          TRISBbits.RB4
#define TRIS_LCD_RS         TRISBbits.RB5
#define TRIS_LCD_BRIGHTNESS TRISBbits.RB3

#define PIN_TEMPERATURE     PORTAbits.AN0
#define TRIS_TEMPERATURE    TRISAbits.RA0

#define PIN_NTC             PORTAbits.AN1
#define TRIS_NTC            TRISAbits.RA1

#define PIN_EB              PORTBbits.RB0
#define PIN_E1              PORTBbits.RB1
#define PIN_E2              PORTBbits.RB2
#define TRIS_EB             TRISBbits.RB0
#define TRIS_E1             TRISBbits.RB1
#define TRIS_E2             TRISBbits.RB2

#define PIN_B1              PORTCbits.RC1
#define TRIS_B1             TRISCbits.RC1
#define PIN_B2              PORTCbits.RC0
#define TRIS_B2             TRISCbits.RC0

#define PIN_USBDETECT       PORTBbits.RB7
#define TRIS_USBDETECT      TRISBbits.RB7

#define PIN_HEATER          PORTCbits.RC2
#define TRIS_HEATER         TRISCbits.RC2

#define heaterON()          {PIN_HEATER = 0;}
#define heaterOFF()         {PIN_HEATER = 1;}

#define INPUT  1
#define OUTPUT 0


#define OpenTEMP()   OpenADC(\
          ADC_FOSC_64 & ADC_RIGHT_JUST & ADC_20_TAD,\
          ADC_CH0 & ADC_INT_OFF & ADC_VREFPLUS_VDD & ADC_VREFMINUS_VSS,\
          13)


#define OpenNTC()   OpenADC(\
          ADC_FOSC_64 & ADC_RIGHT_JUST & ADC_20_TAD,\
          ADC_CH1 & ADC_INT_OFF & ADC_VREFPLUS_VDD & ADC_VREFMINUS_VSS,\
          13)

#define setLCDBackLight(x)  {ClrWdt();SetDCPWM2(x);}
#define setupLCDBackLight() {ClrWdt();OpenPWM2(0xFF);SetDCPWM2(512);}

#define TOOHOT 550
#define maxHeaterPos 100
#define maxPWM 100
#define minPWM 0

#define menuTIMEOUT 5000

#endif